@@ -124,7 +124,7 @@ test_that("parameter plot for iterative search", {
   expect_equal(names(p$data), c(".iter", "name", "value"))
 
   name_vals <- sort(unique(p$data$name))
-  expect_equal(name_vals, c("K", "Piecewise Polynomial Degree"))
+  expect_equal(name_vals, c("K", "Spline Degrees of Freedom"))
   expect_equal(rlang::get_expr(p$mapping$x), expr(.iter))
   expect_equal(rlang::get_expr(p$mapping$y), expr(value))
   expect_equal(p$labels$y, "")
@@ -133,6 +133,7 @@ test_that("parameter plot for iterative search", {
 
 
 test_that("regular grid plot", {
+  skip_if_not_installed("scales", "1.3.0")
   rcv_results <- readRDS(test_path("data", "rcv_results.rds"))
   svm_reg_results <- readRDS(test_path("data", "svm_reg_results.rds"))
 
@@ -166,6 +167,7 @@ test_that("regular grid plot", {
   expect_equal(p$labels$y, "rmse")
   expect_equal(p$labels$x, "deg_free")
 
+
   p <- autoplot(svm_reg_results)
   expect_s3_class(p, "ggplot")
   expect_equal(
@@ -183,7 +185,7 @@ test_that("regular grid plot", {
   expect_equal(p$labels$x, "Cost")
   expect_equal(p$labels$group, "%^*#")
 
-  expect_equal(class(p$scales$scales[[1]]$trans), "trans")
+  expect_true(grepl("^trans", class(p$scales$scales[[1]]$trans)))
   expect_equal(p$scales$scales[[1]]$trans$name, "log-2")
   expect_equal(unique(p$data$name), "Cost")
 })
@@ -193,11 +195,10 @@ test_that("regular grid plot", {
 test_that("coord_obs_pred", {
   data(solubility_test, package = "modeldata")
 
-  library(ggplot2)
   p <-
-    ggplot(solubility_test, aes(x = solubility, y = prediction)) +
-    geom_abline(lty = 2) +
-    geom_point(alpha = 0.5)
+    ggplot2::ggplot(solubility_test, ggplot2::aes(x = solubility, y = prediction)) +
+    ggplot2::geom_abline(lty = 2) +
+    ggplot2::geom_point(alpha = 0.5)
 
   rng <- range(solubility_test[[1]], solubility_test[[2]])
 
@@ -212,9 +213,9 @@ test_that("coord_obs_pred", {
 
   solubility_test$solubility[1] <- NA
   p3 <-
-    ggplot(solubility_test, aes(x = solubility, y = prediction)) +
-    geom_abline(lty = 2) +
-    geom_point(alpha = 0.5)
+    ggplot2::ggplot(solubility_test, ggplot2::aes(x = solubility, y = prediction)) +
+    ggplot2::geom_abline(lty = 2) +
+    ggplot2::geom_point(alpha = 0.5)
   expect_snapshot_warning(print(p3 + coord_obs_pred()))
 })
 
@@ -227,3 +228,129 @@ test_that("1D regular grid x labels", {
     tune_grid(mpg ~ ., resamples = rsample::vfold_cv(mtcars, v = 5), grid = 3)
   expect_equal(autoplot(res)$labels$x, c(cost = "Cost"))
 })
+
+test_that("plot_regular_grid with fairness metrics (#773)", {
+  skip_on_cran()
+  skip_if_not_installed("yardstick", minimum_version = "1.2.0.9001")
+
+  knn <- parsnip::nearest_neighbor("classification", "kknn", neighbors = tune())
+  mtcars_fair <- mtcars
+  mtcars_fair$vs <- as.factor(mtcars_fair$vs)
+  mtcars_fair$cyl <- as.factor(mtcars_fair$cyl)
+  mtcars_fair$am <- as.factor(mtcars_fair$am)
+  set.seed(4400)
+  boots <- rsample::bootstraps(mtcars_fair, 3)
+  n_grid <- 3
+
+  set.seed(1)
+  res <- tune_grid(
+    knn, vs ~ mpg + hp + cyl, resamples = boots, grid = n_grid,
+    metrics =
+      yardstick::metric_set(
+        yardstick::roc_auc,
+        yardstick::demographic_parity(cyl),
+        yardstick::demographic_parity(am)
+      )
+  )
+
+  res_plot <- autoplot(res)
+
+  expect_contains(
+    res_plot$data$.metric,
+    c("demographic_parity(am)", "demographic_parity(cyl)", "roc_auc")
+  )
+})
+
+test_that("plot_marginals with fairness metrics (#773)", {
+  skip_on_cran()
+  skip_if_not_installed("yardstick", minimum_version = "1.2.0.9001")
+
+  knn <- parsnip::nearest_neighbor("classification", "kknn", neighbors = tune(), weight_func = tune())
+  mtcars_fair <- mtcars
+  mtcars_fair$vs <- as.factor(mtcars_fair$vs)
+  mtcars_fair$cyl <- as.factor(mtcars_fair$cyl)
+  mtcars_fair$am <- as.factor(mtcars_fair$am)
+  set.seed(4400)
+  boots <- rsample::bootstraps(mtcars_fair, 3)
+  n_grid <- 3
+
+  set.seed(1)
+  res <- tune_grid(
+    knn, vs ~ mpg + hp + cyl, resamples = boots, grid = n_grid,
+    metrics =
+      yardstick::metric_set(
+        yardstick::roc_auc,
+        yardstick::demographic_parity(cyl),
+        yardstick::demographic_parity(am)
+      )
+  )
+
+  res_plot <- autoplot(res)
+
+  expect_contains(
+    res_plot$data$.metric,
+    c("demographic_parity(am)", "demographic_parity(cyl)", "roc_auc")
+  )
+})
+
+test_that("plot_perf_vs_iter with fairness metrics (#773)", {
+  skip_on_cran()
+  skip_if_not_installed("yardstick", minimum_version = "1.2.0.9001")
+
+  knn <- parsnip::nearest_neighbor("classification", "kknn", neighbors = tune())
+  mtcars_fair <- mtcars
+  mtcars_fair$vs <- as.factor(mtcars_fair$vs)
+  mtcars_fair$cyl <- as.factor(mtcars_fair$cyl)
+  mtcars_fair$am <- as.factor(mtcars_fair$am)
+  set.seed(4400)
+  boots <- rsample::bootstraps(mtcars_fair, 3)
+  n_grid <- 3
+
+  set.seed(1)
+  suppressMessages(
+    res <- tune_bayes(
+      knn, vs ~ mpg + hp + cyl, resamples = boots,
+      metrics =
+        yardstick::metric_set(
+          yardstick::roc_auc,
+          yardstick::demographic_parity(cyl),
+          yardstick::demographic_parity(am)
+        )
+    )
+  )
+
+  res_plot <- autoplot(res, type = "performance")
+
+  expect_contains(
+    res_plot$data$.metric,
+    c("demographic_parity(am)", "demographic_parity(cyl)", "roc_auc")
+  )
+})
+
+test_that("regular grid plot", {
+  skip_if_not_installed("ggplot2", minimum_version = "3.5.0")
+  set.seed(1)
+  res <-
+    parsnip::svm_rbf(cost = tune()) %>%
+    parsnip::set_engine("kernlab") %>%
+    parsnip::set_mode("regression") %>%
+    tune_grid(mpg ~ ., resamples = rsample::vfold_cv(mtcars, v = 5), grid = 1)
+
+  expect_snapshot(
+    error = TRUE,
+    autoplot(res)
+  )
+})
+
+test_that("evaluation time warning for non-survival model", {
+
+  set.seed(1)
+  res <-
+    parsnip::svm_rbf(cost = tune()) %>%
+    parsnip::set_engine("kernlab") %>%
+    parsnip::set_mode("regression") %>%
+    tune_grid(mpg ~ ., resamples = rsample::vfold_cv(mtcars, v = 5), grid = 2)
+
+  expect_snapshot(foo <- autoplot(res, metric = "rmse", eval_time = 10))
+
+})
